#Raspberry Pi Dew Heater control program
#Dec 2019
#Brian Plett
#Uses Pimoroni Explorer pHAT, a temperature sensor, and a relay
#to control a resistor circuit as a dew heater for an all-sky camera
#Pulls outside air temperature and dewpoint from NWS website
#keeps internal temperature 10 degrees above dewpoint
import time
import datetime
import requests
import csv
import os
import explorerhat
#Station ID is closest weather station at the NWS. Go to weather.gov and look up forcast for your area,
#station ID is in () after location name.
settings = {
    'station_ID':'KPDX',
}
#Alternate URL for weather information
#BASE_URL = "http://api.openweathermap.org/data/2.5/weather?appid={0}&zip={1},{2}&units={3}"

#Weather URL to retreive data
BASE_URL = "https://api.weather.gov/stations/{0}/observations/latest"

#delay for relay control, seconds
ControlDelay = 2
A=0
B=0
while True:
    #date to use in log filename
    datestr = datetime.datetime.now().strftime("%Y%m%d")
    #date & time to use for each data row
    localtime = datetime.datetime.now().strftime("%Y/%m/%d %H:%M")
    #CSV file path
    path = '/home/pi/allsky/DewHeaterLogs/DewHeatLog{}.csv'
    while B == 0:
        try:
            #Pull temperature and dewpoint from NWS every 60 seconds
            final_url = BASE_URL.format(settings["station_ID"])
            weather_data = requests.get(final_url, timeout= 5, headers = {'User-agent': 'Raspberry Pi 3+ Allsky Camera user@abcd.com'})
            oatRaw = weather_data.json()["properties"]["temperature"]["value"]
            dewRaw = weather_data.json()["properties"]["dewpoint"]["value"]
            #diagnostic print for raw temperature data
            print(oatRaw, dewRaw)
            OAT = round(oatRaw,3)
            Dew = round(dewRaw,3)
        except:
            A = 0
            B = 1
            break
        A = 0
        B = 1
        break
    if A < 300:
        A = A + ControlDelay
    else:
        B = 0
    #Read raw voltage from Raspberry Pi Explorer PHat and convert to temperature
    T1 = explorerhat.analog.one.read()
    tempC = ((T1*1000)-500)/10
    #tempF = tempC*1.8 +32
    if (tempC < Dew + 10) : V1 = explorerhat.output.two.on()
    if (tempC > Dew + 10.5) : V1 = explorerhat.output.two.off()
    #diagnostic print showing temperatures, dewpoints, and relay output state
    print(' {0:5.2f} degC, {1:5.2f} degC, {2:5.2f} deg C {3:5.0f}'.format(round(OAT,3), round(Dew,3), round(tempC,3), explorerhat.output.two.read()))
    #10 seconds after the minute rolls over, write data to a CSV file
    if A ==10:
        if os.path.isfile(path.format(datestr)):
            print(path.format(datestr))
            with open(path.format(datestr), "a") as csvfile:
                txtwrite = csv.writer(csvfile)
                txtwrite.writerow([localtime,OAT,Dew,tempC, explorerhat.output.two.read()])
        else:
            fieldnames = ['date','Outdoor Air Temp','Dewpoint','Dome Temp','Relay State']
            with open(path.format(datestr), "w") as csvfile:
                txtwrite = csv.writer(csvfile)
                txtwrite.writerow(fieldnames)
                txtwrite.writerow([localtime,OAT,Dew,tempC, explorerhat.output.two.read()])
    time.sleep(ControlDelay)